"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
function composeMiddleware(chain) {
    return function (context, dispatch) {
        return chain.reduceRight(function (res, middleware) {
            return middleware(context)(res);
        }, dispatch);
    };
}
var createReducer = function () {
    var middlewares = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        middlewares[_i] = arguments[_i];
    }
    return function (reducer, initialState, initializer) {
        if (initializer === void 0) { initializer = function (value) { return value; }; }
        var ref = react_1.useRef(initializer(initialState));
        var _a = react_1.useState(ref.current), setState = _a[1];
        var middlewareDispatch = function (_) {
            if (_ === void 0) { _ = {}; }
            throw new Error('Dispatching while constructing your middleware is not allowed. ' +
                'Other middleware would not be applied to this dispatch.');
        };
        var dispatch = function (action) {
            ref.current = reducer(ref.current, action);
            setState(ref.current);
            return action;
        };
        var composedMiddleware = react_1.useMemo(function () {
            return composeMiddleware(middlewares);
        }, middlewares);
        var middlewareAPI = {
            getState: function () { return ref.current; },
            dispatch: function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return middlewareDispatch.apply(void 0, args);
            },
        };
        middlewareDispatch = composedMiddleware(middlewareAPI, dispatch);
        return [ref.current, middlewareDispatch];
    };
};
exports.default = createReducer;
